from rdflib import URIRef, Literal, Namespace, Graph
from rdflib.namespace import RDF


class Report:
    def __init__(self, report_uri, reportType, title, starting_activity,
                 startedAtTime, ending_activity, endedAtTime, reportingSystemJobUri, description=None):
        """
        Creates objects of type PROMS-O Report

        :param reportType: proms:reportType
        :param title: dc:title
        :param description: dc:description
        :param starting_activity: A proms:Entity
        :param startedAtTime: prov:startedAtTime. Must not be a proms:startedAtTimeDiffStep
        :param ending_activity: A proms:Entity
        :param endedAtTime: prov:endedAtTime. Must not be a proms:endedAtTimeDiffStep
        :return: nothing (a Report object is created)
        """

        #region Set instance variables
        if report_uri:
            self.report_uri = report_uri
        else:
            self.report_uri = URIRef('http://promsns.org/report-placeholder-uri')
        self.reportType = reportType
        self.title = title
        self.starting_activity = starting_activity
        self.startedAtTime = startedAtTime
        self.ending_activity = ending_activity
        self.endedAtTime = endedAtTime
        if reportingSystemJobUri:
            self.reportingSystemJobUri = reportingSystemJobUri
        else:
            self.description = None
        if description:
            self.description = description
        else:
            self.description = None
        #endregion

        #region Add instance variables to graph
        self.g = Graph()

        PROMS = Namespace('http://promsns.org/def/proms#')
        PROV = Namespace('http://www.w3.org/ns/prov#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        DC = Namespace('http://purl.org/dc/elements/1.1/')

        self.g = Graph()
        self.g.add((URIRef(self.report_uri), RDF.type, PROMS.Report))
        self.g.add((URIRef(self.report_uri), PROMS.reportType, URIRef(self.reportType)))
        self.g.add((URIRef(self.report_uri), DC.title, Literal(self.title, datatype=XSD.string)))

        #add the Activity to the graph
        self.g = self.g + self.starting_activity.get_graph()
        #link the Activity to the Report
        self.g.add((URIRef(self.report_uri),
                    PROMS.startingActivity,
                    URIRef(self.starting_activity.get_id())))

        self.g.add((URIRef(self.report_uri),
                    PROV.startedAtTime,
                    Literal(self.startedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))

        #add the Activity to the graph
        self.g = self.g + self.ending_activity.get_graph()
        #link the Activity to the Report
        self.g.add((URIRef(self.report_uri),
                    PROMS.endingActivity,
                    URIRef(self.ending_activity.get_id())))

        self.g.add((URIRef(self.report_uri),
                    PROV.endedAtTime,
                    Literal(self.endedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))

        if self.description:
            self.g.add((URIRef(self.report_uri),
                        PROMS.reportingSystemJobUri,
                        URIRef(self.reportingSystemJobUri)))

        if self.description:
            self.g.add((URIRef(self.report_uri),
                        DC.description,
                        Literal(self.description, datatype=XSD.string)))
        #endregion

        return

    def get_id(self):
        """
        Get the node URI of this Report. Always a URI.

        :return: A URI
        """
        return URIRef(self.report_uri)

    def get_graph(self):
        """
        Generates the RDF graph of this Report

        :return: This Report's RDF graph according to PROMS-O
        """
        return self.g


class ReportType:
    """
    This class specifies acceptable URI values for the proms:reportType
    property of PROMS-O Reports.
    """
    Basic = 'http://promsns.org/def/proms#Basic'
    External = 'http://promsns.org/def/proms#External'
    Internal = 'http://promsns.org/def/proms#Internal'



import rdflib
from rdflib import URIRef, BNode, Literal, Namespace, Graph
from rdflib.namespace import RDF
import datetime

class Entity:
    def __init__(self, entity_uri, title, created, creator, description=None,
                 rights=None, confidentialityStatus=None, metadataUri=None, downloadURL=None):
        """
        Creates objects of type PROMS-O Entity

        :param entity_uri: A URI for this Entity. Must be either a URI or a URIRef(BNode()) or None. If None, a BNode will be created for the Entity.
        :param title: dc:title
        :param description: dc:description
        :param created: dc:created
        :param creator: dc:creator
        :param rights: dc:rights
        :param confidentialityStatus: The confidentiality status of this Entity
        :param metadataUri: A URI leading to this Entity's metadata document (expected to be a document conformant to a schema such as ANZLIC or ISO19115)
        :param downloadURL: A URL leading to this Entity's data
        :return: nothing (an Entity object is created)
        """

        #region Set instance variables
        if entity_uri is None:
            self.entity_uri = BNode()
        else:
            #it is a URIRef or a BNode elsewhere specified
            self.entity_uri = entity_uri

        if type(self.entity_uri) == rdflib.BNode:
            self.title = title
            self.created = created
            self.creator = creator
            if description:
                self.description = description
            if rights:
                self.license = rights
            else:
                self.license = None                
            if confidentialityStatus:
                self.confidentialityStatus = confidentialityStatus
            else:
                self.confidentialityStatus = None                  
            if metadataUri:
                self.metadataUri = metadataUri
            else:
                self.metadataUri = None
            if downloadURL:
                self.downloadURL = downloadURL
            else:
                self.downloadURL = None
        #endregion Set instance variables

        #region Add instance variables to graph
        self.g = Graph()

        PROV = Namespace('http://www.w3.org/ns/prov#')

        self.g.add((URIRef(self.entity_uri),
                    RDF.type,
                    PROV.Entity))

        if type(self.entity_uri) == rdflib.BNode:
            XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
            PROMS = Namespace('http://promsns.org/def/proms#')
            DC = Namespace('http://purl.org/dc/elements/1.1/')
            DCAT = Namespace('http://www.w3.org/ns/dcat#')

            self.g.add((URIRef(self.entity_uri),
                        DC.title,
                        Literal(self.title, datatype=XSD.string)))

            if isinstance(self.created, datetime.datetime):
                self.g.add((URIRef(self.entity_uri),
                            DC.created,
                            Literal(self.created.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))
            else:
                if isinstance(self.created, int):
                   self.g.add((URIRef(self.entity_uri),
                               PROMS.createdAtTimeDiffStep,
                               Literal(self.created, datatype=XSD.integer)))

            self.g.add((URIRef(self.entity_uri),
                        DC.creator,
                        Literal(self.creator, datatype=XSD.string)))

            if self.description:
                self.g.add((URIRef(self.entity_uri),
                            DC.description,
                            Literal(self.description, datatype=XSD.string)))

            if self.license:
                self.g.add((URIRef(self.entity_uri),
                            DC.license,
                            URIRef(self.license)))

            if self.confidentialityStatus:
                self.g.add((URIRef(self.entity_uri),
                            PROMS.confidentialityStatus,
                            URIRef(self.confidentialityStatus)))

            if self.metadataUri:
                self.g.add((URIRef(self.entity_uri),
                            PROMS.metadataUri,
                            Literal(self.metadataUri, datatype=XSD.anyUri)))

            if self.downloadURL:
                self.g.add((URIRef(self.entity_uri),
                            DCAT.downloadURL,
                            Literal(self.downloadURL, datatype=XSD.anyUri)))
        #endregion

        return

    def get_id(self):
        """
        Get the node URI of this Entity, whether a BNode or URI.

        :return: Either a BNode or a URI
        """
        return URIRef(self.entity_uri)

    def get_graph(self):
        """
        Generates the RDF graph of this Entity

        :return: This Entity's RDF graph according to PROMS-O
        """
        return self.g


class ConfidentialityStatus:
    """
    This class specifies acceptable URI values for
    the proms:confidentialityStatus property of PROMS-O Entities.
    """
    PublicDomain = 'http://promsns.org/def/proms/confidentiality/publicdomain'
    Private = 'http://promsns.org/def/proms/confidentiality/private'
    AccessAvailableOnRequest = 'http://promsns.org/def/proms/confidentiality/accessavailableonrequest'
    OwnerDepartmentOnly = 'http://promsns.org/def/proms/confidentiality/ownerdepartmentonly'
    IndigenousSensitive = 'http://promsns.org/def/proms/confidentiality/indigenoussensitive'



import requests


class Reporter:
    def post(self, report_register_uri, report):
        """
        POSTS an RDF-serialised Report object to a PROMS server instance

        :param report_register_uri: the URI of the PROMS server instance's Report register.
        :param report: a pyproms Report class object

        :return: a requests module Response class
        """
        #ensure Reports register has a trailing slash
        if not report_register_uri.endswith('/'):
            report_register_uri += '/'

        #get the RDF version of the Report
        report_str = report.get_graph().serialize(format='n3').decode('utf-8')

        #POST the Report to PROMS
        headers = {'Content-type': 'text/turtle'}
        r = requests.post(report_register_uri, data=report_str, headers=headers)

        return r



from rdflib import URIRef, BNode, Literal, Namespace, Graph
from rdflib.namespace import RDF
import datetime


class Activity:
    def __init__(self, activity_uri, title, wasAttributedTo, startedAtTime, endedAtTime,
                 description=None, used_entities=None, generated_entities=None, wasInformedBy=None):
        """
        Creates objects of type PROMS-O Activity

        :param activity_uri: A URI for this Activity. Must be either a URI or a None. If None, a BNode will be created for the Activity.
        :param title: dc:title
        :param description: dc:description
        :param wasAttributedTo: prov:wasAttributedTo
        :param startedAtTime: prov:startedAtTime or proms:startedAtTimeDiffStep
        :param endedAtTime: prov:endedAtTime or proms:endedAtTimeDiffStep
        :param wasInformedBy: prov:wasInformedBy
        :param used_entities: List of prov:Entities
        :param generated_entities: List of prov:Entities
        :return: nothing (an initialised Activity object)
        """

        #region Set instance variables
        if activity_uri:
            self.activity_uri = activity_uri
        else:
            self.activity_uri = BNode()

        self.title = title
        self.wasAttributedTo = wasAttributedTo
        self.startedAtTime = startedAtTime
        self.endedAtTime = endedAtTime

        if description:
            self.description = description
        else:
            self.description = None

        if used_entities:
            self.used_entities = used_entities
        else:
            self.used_entities = None

        if generated_entities:
            self.generated_entities = generated_entities
        else:
            self.generated_entities = None

        if wasInformedBy:
            self.wasInformedBy = wasInformedBy
        else:
            self.wasInformedBy = None
        #endregion

        #region Add instance variables to graph
        self.g = Graph()

        PROV = Namespace('http://www.w3.org/ns/prov#')
        PROMS = Namespace('http://promsns.org/def/proms#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        DC = Namespace('http://purl.org/dc/elements/1.1/')

        self.g.add((URIRef(self.activity_uri),
                    RDF.type,
                    PROV.Activity))

        self.g.add((URIRef(self.activity_uri),
                    DC.title,
                    Literal(self.title)))

        self.g.add((URIRef(self.activity_uri),
                    PROV.wasAttributedTo,
                    URIRef(self.wasAttributedTo)))

        if isinstance(self.startedAtTime, datetime.datetime):
            #add an absolute time
            self.g.add((URIRef(self.activity_uri),
                        PROV.startedAtTime,
                        Literal(self.startedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))
            #add a relative timestep integer
            self.g.add((URIRef(self.activity_uri),
                        PROMS.startedAtTimeDiffStep,
                        Literal(self.startedAtTime, datatype=XSD.integer)))

        if isinstance(self.endedAtTime, datetime.datetime):
            #add an absolute timestep integer
            self.g.add((URIRef(self.activity_uri),
                        PROV.endedAtTime,
                        Literal(self.endedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))
            #add a relative time
            self.g.add((URIRef(self.activity_uri),
                        PROMS.endedAtTimeDiffStep,
                        Literal(self.endedAtTime, datatype=XSD.integer)))

        if self.description:
            self.g.add((URIRef(self.activity_uri),
                        DC.description,
                        Literal(self.description)))

        if self.used_entities:
            for used_entity in self.used_entities:
                #add the Entity to the graph
                self.g = self.g + used_entity.get_graph()
                #associate the Entity with the Activity
                self.g.add((URIRef(self.activity_uri),
                            PROV.used,
                            URIRef(used_entity.get_id())))

        if self.generated_entities:
            for generated_entity in self.generated_entities:
                #add the Entity to the graph
                self.g = self.g + generated_entity.get_graph()
                #associate the Entity with the Activity
                self.g.add((URIRef(self.activity_uri),
                            PROV.generated,
                            URIRef(generated_entity.get_id())))

        if self.wasInformedBy:
            self.g.add((URIRef(self.activity_uri),
                        PROV.wasInformedBy,
                        URIRef(self.wasInformedBy)))
        #endregion

        return

    def get_id(self):
        """
        Get the node URI of this Activity, whether a BNode or URI.

        :return: Either a BNode or a URI
        """
        return URIRef(self.activity_uri)

    def get_graph(self):
        """
        Generates the RDF graph of this Activity

        :return: This Activity's RDF graph according to PROMS-O
        """
        return self.g






from setuptools import setup
#from distutils.core import setup

setup(
    name = 'pyproms',
    packages = ['pyproms'],
    version = '1.1.2',
    author = 'Nicholas Car',
    author_email = 'nicholas.car@csiro.au',
    url = 'https://github.com/ncar/pyproms',
    download_url = 'https://github.com/ncar/pyproms/tarball/1.1.2',
    license = 'LICENSE.txt',
    description = 'PyPROMS is a client library to make PROV and PROMS documents and submit them to a PROMS server instance',
    keywords = ['rdf', 'prov', 'prov-o', 'proms', 'proms-o', 'provenance'],
    long_description = '''
PyProms is a library of functions that helps Python coders generate
provenance reports for their scripts using the PROV and PROMS Ontologies.
It also has a function to send a Report to and instances of the PROMS
Server.

Within the library are a series of classes that match the PROMS-O OWL
classes, for example the Python class Report matches the PROMS-O Report
class.

PyProms is expected to be extended so if you need so add to it to get your
reporting job done, please do so! If you're really nice, you might consider
letting the author, Nicholas Car <nicholas.car@csiro.au> know what you've
added as this will help with further releases.
    ''',
    install_requires = ['rdflib >= 3.0.0'],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)'
    ],
)

#use http://peterdowns.com/posts/first-time-with-pypi.html